﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using WCS.Entity.fj;
using wms.dto.request.fj;
using wms.dto.request.hj;
using wms.dto.request.share;
using wms.dto.response;
using wms.dto.response.fj;
using wms.service.Service;
using wms.sqlsugar.model;
using wms.sqlsugar.model.fj;

namespace wms.service.IService
{
    /// <summary>
    ///  分拣服务接口
    /// </summary>
    public interface IFJService
    {
        public string GetSysConfigContentByCode(string code);

        public bool SyncMaterInfo(List<FJSyncMaterInfoResponseItem> ResData);

        public bool SyncMaterInfoLX(List<SxSyncMaterInfoListRequestItem> ResData);

        public SRes WetFormulaInfo(WetFormulaInfoRequest reqDto);

        public SRes SyncMachineInfo(SyncMachineInfoRequest ResData);

        public fjSysConfig GetSysConfigByCode(string code);

        public fjSysConfig GetSysConfigByExpression(Expression<Func<fjSysConfig, bool>> WhereExpression);

        public List<BillPushinfo> GetBillPushinfoList(Expression<Func<BillPushinfo, bool>> WhereExpression, bool type = false);

        public bool UpdatePushModelColumns(UpdateModelColumns<BillPushinfo> updateModel);

        public bool UpdateSysConfigModelColumns(UpdateModelColumns<fjSysConfig> updateModel);

        List<BaseMatinfo> GetBaseMatinfoList(Expression<Func<BaseMatinfo, bool>> WhereExpression);

        bool UpdateMaterModelColumns(UpdateModelColumns<BaseMatinfo> updateModel);

        public bool UpdateMaterModel(BaseMatinfo updateModel);

        public bool UpdateBillPushinfoModelColumns(UpdateModelColumns<BillPushinfo> updateModel);

        public SRes CurtainProductionOrder(CurtainProductionOrderRequest reqDto);

        public SRes SpoolMaterialInfoTrans(SpoolMaterialInfoTransRequest reqDto);

        public SRes ManualBuildStockTrans(ManualBuildStockTransRequest reqDto);

        /// <summary>
        ///  江景获取流向
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes<string> AllocatedSpoolFlow(AllocatedSpoolFlowRequest reqDto);

        /// <summary>
        ///  MES获取流向异常信息
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes QueryFlowDirectionExceptions(GetFlowExceptionResultsRequest reqDto);

        public SRes ManualBuildEmptyStock(ManualBuildEmptyStockRequest reqDto);

        public AgvCallbackResponse AgvCallback(AgvCallbackRequest reqDto);

        public SRes CurtainProductionStockOut(CurtainProductionStockOutRequest reqDto);

        /// <summary>
        /// 创建货位,用于少数虚拟货位及平库
        /// </summary>
        /// <param name="WarehouseCode">仓库号</param>
        /// <param name="Code">货位号</param>
        /// <returns></returns>
        public SRes CreatWarecell(string WarehouseCode, string Code);

        /// <summary>
        ///  帘线叫料接口(人工码垛托盘)
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes CurtainProductionStockOut1(CurtainProductionStockOutRequest1 reqDto);

        public SRes ManualCurtainProductionStockOut(ManualCurtainProductionStockOut reqDto);

        public bool AllocationWarehouseSort(AllocationWarehouseSortRequest reqDto);

        public SRes ProductionBack(FjProductionBackRequest reqDto);

        public SRes ProductionLeftFeedBack(string doccode);

        /// <summary>
        ///  人工创建满托入库任务
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes CreateMaterialPalletWarehouTask(CreateMaterialPalletWarehouTaskRequest reqDto);

        /// <summary>
        ///  人工创建UT满托入库任务
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes CreateMaterialUTPalletWarehouTask(CreateMaterialPalletWarehouTaskRequest reqDto);
        

        /// <summary>
        ///  芯股满托创建库存信息
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes CreateMaterialPalletWarehouTask1(CreateMaterialPalletWarehouTask1Request reqDto);

        /// <summary>
        ///  重绕区组盘信息
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes CreateMaterialPalletWarehouTask2(RewindFullPalletTaskRequest reqDto);

        /// <summary>
        ///  创建退料任务
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes MaterialReturnTaskCreator(MaterialReturnTaskCreatorRequest reqDto);

        public SRes ErrorMaterialStockOut(FjErrorMaterialStockOutRequest reqDto);

        public SRes BGradeMaterialStockOut(BGradeMaterialStockOutRequest reqDto);

        public SRes StockChange(FjStockChangeRequest reqDto);

        public SRes BomInfoTrans(BomInfoTransRequest reqDto);

        public SRes CurtainOrderStartCheck(CurtainOrderStartCheckRequest reqDto);

        public GetFJCurStockListResponse GetCurStockList(GetFJCurStockListRequest reqDto);

        /// <summary>
        /// 创建芯股用空托入库任务
        /// </summary>
        public void CreateCoreStrandEmptyPalletEnterWarehouse();

        /// <summary>
        /// 创建芯股站台用的空托出库任务
        /// </summary>
        /// <returns></returns>
        public SRes CoreStrandSupportReplenishEmptyPallet();

        /// <summary>
        ///  变更流向
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes EditorialFlow(FjEditorialFlowRequest reqDto);

        /// <summary>
        ///  分拣北自动生成库位
        /// </summary>
        /// <returns></returns>
        public SRes FJ1NorthAutoGenerateLocation();

        /// <summary>
        /// 获取垛型数组
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        public SRes<List<BillBomsetgrp>> FjGetSetGrpList(FjGetSetGrpListRequest req);

        /// <summary>
        ///  分拣南自动生成库位
        /// </summary>
        /// <returns></returns>
        public SRes FJSouthAutoGenerateLocation();

        /// <summary>
        ///  分拣北环形库自动生成库位
        /// </summary>
        /// <returns></returns>
        public SRes RingLibraryAutoGenerateLocation(string WareCode);

        /// <summary>
        ///  环形库申请货位
        /// </summary>
        /// <returns></returns>
        public SRes<FJApplyStockInLocResponse> RingApplyStockInLoc(FJApplyStockInLocRequest reqEntity);

        /// <summary>
        ///  一楼空托盘人工入库任务创建
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes OneFloorWorkerBuildEmptyPalletsStock(FJBuildEmptyPalletsStockRequest reqDto);

        /// <summary>
        ///  申请货位
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes<FJApplyStockInLocResponse> ApplyStockInLoc(FJApplyStockInLocRequest reqDto);

        /// <summary>
        ///  平库申请货位
        /// </summary>
        /// <param name="reqEntity"></param>
        /// <returns></returns>
        public SRes<FJApplyStockInLocResponse> FindAvailableFlatStorageSlot(FJApplyStockInLocRequest reqEntity);

        /// <summary>
        ///  手动出库
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public string ManualTask(FJManualTaskRequest reqDto);

        /// <summary>
        ///  完成任务
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes<int> CompleteTask(CompleteTaskRequest reqDto);

        /// <summary>
        ///  取消任务
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes<int> CancelTask(CompleteTaskRequest reqDto);

        /// <summary>
        ///  工字轮/芯股进入主线扫码
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns
        public SRes<SRes> EnteMainLine(FJEnteMainLineRequest reqDto);

        /// <summary>
        ///  申请空托盘组出库任务
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes ApplyEmptyPalletGroupOutStorage(FjApplyEmptyPalletGroupOutStorageRequest reqDto);

        /// <summary>
        ///   生成组盘任务，空托盘
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes GroupTask(FJBuildEmptyPalletsStockDto reqDto);

        /// <summary>
        ///   生成组盘任务，空托盘
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes BingPallet(FJBingPalletDto reqDto);

        /// <summary>
        ///   生成组盘任务，空托盘
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes<int> GetTwoPallet(FJGetTwoPalletDto reqDto);

        /// <summary>
        ///   生成组盘任务，空托盘
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes ApplyPalletizingStockOut(RingApplyPalletizingStockOutRequest reqDto);

        /// <summary>
        ///  堆垛机出库完成
        /// </summary>
        /// <param name="reqDto"></param>
        /// <returns></returns>
        public SRes<int> SrmPickOutCompleted(SrmPickOutCompletedRequest reqDto);

        public SRes PalletizingCreateseErrorTasks(PalletizingCreateseErrorTasksRequest reqDto);

        fjSysJob GetSysJobEntity(DetailRequest2Str req);

        fjSysJob GetSysJobEntityByCode(string Code);

        List<fjSysJobApiRelation> GetSysApiDetail(long Id);

        List<fjSysJob> GetJobList();

        /// <summary>
        /// 查询任务  通过条码查询任务
        /// </summary>
        /// <param name="barCode">条码</param>
        /// <returns></returns>
        public SRes<WCS_TaskOld> GetTaskInfo(FJGetTaskInfoRequest dto);
    }

    public class FJGetTaskInfoRequest
    {
        /// <summary>
        ///  条码
        /// </summary>
        public string BarCode { get; set; }
    }
}